@@ -1,10 +1,16 @@
+/* $Id: network_udp.c 5331 2006-06-21 19:39:54Z tron $ */
+
 #include "stdafx.h"
+#include "debug.h"
+#include "string.h"
 #include "network_data.h"
 
 #ifdef ENABLE_NETWORK
 
+#include "map.h"
 #include "network_gamelist.h"
 #include "network_udp.h"
+#include "variables.h"
 
 extern void UpdateNetworkGameWindow(bool unselect);
 extern void NetworkPopulateCompanyInfo(void);
@@ -29,13 +35,15 @@ typedef enum {
 } PacketUDPType;
 
 enum {
-	ADVERTISE_NORMAL_INTERVAL = 450,	// interval between advertising in days
-	ADVERTISE_RETRY_INTERVAL = 5,			// readvertise when no response after this amount of days
-	ADVERTISE_RETRY_TIMES = 3					// give up readvertising after this much failed retries
+	ADVERTISE_NORMAL_INTERVAL = 30000, // interval between advertising in ticks (15 minutes)
+	ADVERTISE_RETRY_INTERVAL = 300,    // readvertise when no response after this many ticks (9 seconds)
+	ADVERTISE_RETRY_TIMES = 3          // give up readvertising after this much failed retries
 };
 
 #define DEF_UDP_RECEIVE_COMMAND(type) void NetworkPacketReceive_ ## type ## _command(Packet *p, struct sockaddr_in *client_addr)
-void NetworkSendUDP_Packet(SOCKET udp, Packet *p, struct sockaddr_in *recv);
+static void NetworkSendUDP_Packet(SOCKET udp, Packet* p, struct sockaddr_in* recv);
+
+static NetworkClientState _udp_cs;
 
 DEF_UDP_RECEIVE_COMMAND(PACKET_UDP_CLIENT_FIND_SERVER)
 {
@@ -48,16 +56,25 @@ DEF_UDP_RECEIVE_COMMAND(PACKET_UDP_CLIENT_FIND_SERVER)
 
 	// Update some game_info
 	_network_game_info.game_date = _date;
+	_network_game_info.map_width = MapSizeX();
+	_network_game_info.map_height = MapSizeY();
 	_network_game_info.map_set = _opt.landscape;
 
 	NetworkSend_uint8 (packet, NETWORK_GAME_INFO_VERSION);
+
+	/* NETWORK_GAME_INFO_VERSION = 2 */
+	NetworkSend_uint8 (packet, _network_game_info.companies_max);
+	NetworkSend_uint8 (packet, ActivePlayerCount());
+	NetworkSend_uint8 (packet, _network_game_info.spectators_max);
+
+	/* NETWORK_GAME_INFO_VERSION = 1 */
 	NetworkSend_string(packet, _network_game_info.server_name);
 	NetworkSend_string(packet, _network_game_info.server_revision);
 	NetworkSend_uint8 (packet, _network_game_info.server_lang);
 	NetworkSend_uint8 (packet, _network_game_info.use_password);
 	NetworkSend_uint8 (packet, _network_game_info.clients_max);
 	NetworkSend_uint8 (packet, _network_game_info.clients_on);
-	NetworkSend_uint8 (packet, _network_game_info.spectators_on);
+	NetworkSend_uint8 (packet, NetworkSpectatorCount());
 	NetworkSend_uint16(packet, _network_game_info.game_date);
 	NetworkSend_uint16(packet, _network_game_info.start_date);
 	NetworkSend_string(packet, _network_game_info.map_name);
@@ -76,6 +93,7 @@ DEF_UDP_RECEIVE_COMMAND(PACKET_UDP_CLIENT_FIND_SERVER)
 
 DEF_UDP_RECEIVE_COMMAND(PACKET_UDP_SERVER_RESPONSE)
 {
+	extern const char _openttd_revision[];
 	NetworkGameList *item;
 	byte game_info_version;
 
@@ -83,32 +101,51 @@ DEF_UDP_RECEIVE_COMMAND(PACKET_UDP_SERVER_RESPONSE)
 	if (_network_udp_server)
 		return;
 
-	game_info_version = NetworkRecv_uint8(p);
+	game_info_version = NetworkRecv_uint8(&_udp_cs, p);
 
+	if (_udp_cs.quited) return;
 
 	DEBUG(net, 6)("[NET][UDP] Server response from %s:%d", inet_ntoa(client_addr->sin_addr),ntohs(client_addr->sin_port));
 
 	// Find next item
 	item = NetworkGameListAddItem(inet_addr(inet_ntoa(client_addr->sin_addr)), ntohs(client_addr->sin_port));
 
-	if (game_info_version == 1) {
-		NetworkRecv_string(p, item->info.server_name, sizeof(item->info.server_name));
-		NetworkRecv_string(p, item->info.server_revision, sizeof(item->info.server_revision));
-		item->info.server_lang = NetworkRecv_uint8(p);
-		item->info.use_password = NetworkRecv_uint8(p);
-		item->info.clients_max = NetworkRecv_uint8(p);
-		item->info.clients_on = NetworkRecv_uint8(p);
-		item->info.spectators_on = NetworkRecv_uint8(p);
-		item->info.game_date = NetworkRecv_uint16(p);
-		item->info.start_date = NetworkRecv_uint16(p);
-		NetworkRecv_string(p, item->info.map_name, sizeof(item->info.map_name));
-		item->info.map_width = NetworkRecv_uint16(p);
-		item->info.map_height = NetworkRecv_uint16(p);
-		item->info.map_set = NetworkRecv_uint8(p);
-		item->info.dedicated = NetworkRecv_uint8(p);
+	/* Please observer the order. In the order in which packets are sent
+	 * they are to be received */
+	switch (game_info_version) {
+		case 2:
+			item->info.companies_max = NetworkRecv_uint8(&_udp_cs, p);
+			item->info.companies_on = NetworkRecv_uint8(&_udp_cs, p);
+			item->info.spectators_max = NetworkRecv_uint8(&_udp_cs, p);
+			/* Fallthrough */
+		case 1:
+			NetworkRecv_string(&_udp_cs, p, item->info.server_name, sizeof(item->info.server_name));
+			NetworkRecv_string(&_udp_cs, p, item->info.server_revision, sizeof(item->info.server_revision));
+			item->info.server_lang   = NetworkRecv_uint8(&_udp_cs, p);
+			item->info.use_password  = NetworkRecv_uint8(&_udp_cs, p);
+			item->info.clients_max   = NetworkRecv_uint8(&_udp_cs, p);
+			item->info.clients_on    = NetworkRecv_uint8(&_udp_cs, p);
+			item->info.spectators_on = NetworkRecv_uint8(&_udp_cs, p);
+			item->info.game_date     = NetworkRecv_uint16(&_udp_cs, p);
+			item->info.start_date    = NetworkRecv_uint16(&_udp_cs, p);
+			NetworkRecv_string(&_udp_cs, p, item->info.map_name, sizeof(item->info.map_name));
+			item->info.map_width     = NetworkRecv_uint16(&_udp_cs, p);
+			item->info.map_height    = NetworkRecv_uint16(&_udp_cs, p);
+			item->info.map_set       = NetworkRecv_uint8(&_udp_cs, p);
+			item->info.dedicated     = NetworkRecv_uint8(&_udp_cs, p);
 
-		if (item->info.hostname[0] == '\0')
-			snprintf(item->info.hostname, sizeof(item->info.hostname), "%s", inet_ntoa(client_addr->sin_addr));
+			if (item->info.server_lang >= NETWORK_NUM_LANGUAGES) item->info.server_lang = 0;
+			if (item->info.map_set >= NUM_LANDSCAPE ) item->info.map_set = 0;
+
+			if (item->info.hostname[0] == '\0')
+				snprintf(item->info.hostname, sizeof(item->info.hostname), "%s", inet_ntoa(client_addr->sin_addr));
+
+			/* Check if we are allowed on this server based on the revision-match */
+			item->info.compatible = (
+			strncmp(item->info.server_revision, _openttd_revision, NETWORK_REVISION_LENGTH) == 0 ||
+			strncmp(item->info.server_revision, NOREV_STRING, NETWORK_REVISION_LENGTH) == 0) ? true : false;
+
+			break;
 	}
 
 	item->online = true;
@@ -122,7 +159,6 @@ DEF_UDP_RECEIVE_COMMAND(PACKET_UDP_CLIENT_DETAIL_INFO)
 	NetworkClientInfo *ci;
 	Packet *packet;
 	Player *player;
-	byte active = 0;
 	byte current = 0;
 	int i;
 
@@ -132,14 +168,9 @@ DEF_UDP_RECEIVE_COMMAND(PACKET_UDP_CLIENT_DETAIL_INFO)
 
 	packet = NetworkSend_Init(PACKET_UDP_SERVER_DETAIL_INFO);
 
-	FOR_ALL_PLAYERS(player) {
-		if (player->is_active)
-			active++;
-	}
-
 	/* Send the amount of active companies */
 	NetworkSend_uint8 (packet, NETWORK_COMPANY_INFO_VERSION);
-	NetworkSend_uint8 (packet, active);
+	NetworkSend_uint8 (packet, ActivePlayerCount());
 
 	/* Fetch the latest version of everything */
 	NetworkPopulateCompanyInfo();
@@ -162,6 +193,13 @@ DEF_UDP_RECEIVE_COMMAND(PACKET_UDP_CLIENT_DETAIL_INFO)
 		NetworkSend_uint64(packet, _network_player_info[player->index].income);
 		NetworkSend_uint16(packet, _network_player_info[player->index].performance);
 
+                /* Send 1 if there is a passord for the company else send 0 */
+		if (_network_player_info[player->index].password[0] != '\0') {
+			NetworkSend_uint8 (packet, 1);
+		} else {
+			NetworkSend_uint8 (packet, 0);
+		}
+
 		for (i = 0; i < NETWORK_VEHICLE_TYPES; i++)
 			NetworkSend_uint16(packet, _network_player_info[player->index].num_vehicle[i]);
 
@@ -234,12 +272,15 @@ DEF_UDP_RECEIVE_COMMAND(PACKET_UDP_MASTER_RESPONSE_LIST) {
 	 * an uint32 (ip) and an uint16 (port) for each pair
 	 */
 
-	ver = NetworkRecv_uint8(p);
+	ver = NetworkRecv_uint8(&_udp_cs, p);
+
+	if (_udp_cs.quited)
+		return;
 
 	if (ver == 1) {
-		for (i = NetworkRecv_uint16(p); i != 0 ; i--) {
-			ip.s_addr = TO_LE32(NetworkRecv_uint32(p));
-			port = NetworkRecv_uint16(p);
+		for (i = NetworkRecv_uint16(&_udp_cs, p); i != 0 ; i--) {
+			ip.s_addr = TO_LE32(NetworkRecv_uint32(&_udp_cs, p));
+			port = NetworkRecv_uint16(&_udp_cs, p);
 			NetworkUDPQueryServer(inet_ntoa(ip), port);
 		}
 	}
@@ -275,22 +316,30 @@ static NetworkUDPPacket* const _network_udp_packet[] = {
 assert_compile(lengthof(_network_udp_packet) == PACKET_UDP_END);
 
 
-void NetworkHandleUDPPacket(Packet *p, struct sockaddr_in *client_addr)
+static void NetworkHandleUDPPacket(Packet* p, struct sockaddr_in* client_addr)
 {
 	byte type;
 
-	type = NetworkRecv_uint8(p);
+	/* Fake a client, so we can see when there is an illegal packet */
+	_udp_cs.socket = INVALID_SOCKET;
+	_udp_cs.quited = false;
 
-	if (type < PACKET_UDP_END && _network_udp_packet[type] != NULL) {
+	type = NetworkRecv_uint8(&_udp_cs, p);
+
+	if (type < PACKET_UDP_END && _network_udp_packet[type] != NULL && !_udp_cs.quited) {
 		_network_udp_packet[type](p, client_addr);
-	}	else {
-		DEBUG(net, 0)("[NET][UDP] Received invalid packet type %d", type);
+	} else {
+		if (!_udp_cs.quited) {
+			DEBUG(net, 0)("[NET][UDP] Received invalid packet type %d", type);
+		} else {
+			DEBUG(net, 0)("[NET][UDP] Received illegal packet");
+		}
 	}
 }
 
 
 // Send a packet over UDP
-void NetworkSendUDP_Packet(SOCKET udp, Packet *p, struct sockaddr_in *recv)
+static void NetworkSendUDP_Packet(SOCKET udp, Packet* p, struct sockaddr_in* recv)
 {
 	int res;
 
@@ -324,7 +373,11 @@ bool NetworkUDPListen(SOCKET *udp, uint32 host, uint16 port, bool broadcast)
 	// set nonblocking mode for socket
 	{
 		unsigned long blocking = 1;
+#ifndef BEOS_NET_SERVER
 		ioctlsocket(*udp, FIONBIO, &blocking);
+#else
+		setsockopt(*udp, SOL_SOCKET, SO_NONBLOCK, &blocking, NULL);
+#endif
 	}
 
 	sin.sin_family = AF_INET;
@@ -340,7 +393,9 @@ bool NetworkUDPListen(SOCKET *udp, uint32 host, uint16 port, bool broadcast)
 	if (broadcast) {
 		/* Enable broadcast */
 		unsigned long val = 1;
+#ifndef BEOS_NET_SERVER // will work around this, some day; maybe.
 		setsockopt(*udp, SOL_SOCKET, SO_BROADCAST, (char *) &val , sizeof(val));
+#endif
 	}
 
 	DEBUG(net, 1)("[NET][UDP] Listening on port %s:%d", inet_ntoa(*(struct in_addr *)&host), port);
@@ -354,17 +409,23 @@ void NetworkUDPClose(void)
 	DEBUG(net, 1) ("[NET][UDP] Closed listeners");
 
 	if (_network_udp_server) {
-		closesocket(_udp_server_socket);
-		_udp_server_socket = INVALID_SOCKET;
+		if (_udp_server_socket != INVALID_SOCKET) {
+			closesocket(_udp_server_socket);
+			_udp_server_socket = INVALID_SOCKET;
+		}
 
-		closesocket(_udp_master_socket);
-		_udp_master_socket = INVALID_SOCKET;
+		if (_udp_master_socket != INVALID_SOCKET) {
+			closesocket(_udp_master_socket);
+			_udp_master_socket = INVALID_SOCKET;
+		}
 
 		_network_udp_server = false;
 		_network_udp_broadcast = 0;
 	} else {
-		closesocket(_udp_client_socket);
-		_udp_client_socket = INVALID_SOCKET;
+		if (_udp_client_socket != INVALID_SOCKET) {
+			closesocket(_udp_client_socket);
+			_udp_client_socket = INVALID_SOCKET;
+		}
 		_network_udp_broadcast = 0;
 	}
 }
@@ -373,11 +434,7 @@ void NetworkUDPClose(void)
 void NetworkUDPReceive(SOCKET udp)
 {
 	struct sockaddr_in client_addr;
-#ifndef __MORPHOS__
-	int client_len;
-#else
-	LONG client_len; // for some reason we need a 'LONG' under MorphOS
-#endif
+	socklen_t client_len;
 	int nbytes;
 	static Packet *p = NULL;
 	int packet_len;
@@ -411,35 +468,29 @@ void NetworkUDPReceive(SOCKET udp)
 }
 
 // Broadcast to all ips
-void NetworkUDPBroadCast(SOCKET udp)
+static void NetworkUDPBroadCast(SOCKET udp)
 {
 	int i;
 	struct sockaddr_in out_addr;
-	byte *bcptr;
-	uint32 bcaddr;
 	Packet *p;
 
 	// Init the packet
 	p = NetworkSend_Init(PACKET_UDP_CLIENT_FIND_SERVER);
 
-	// Go through all the ips on this pc
 	i = 0;
-	while (_network_ip_list[i] != 0) {
-		bcaddr = _network_ip_list[i];
-		bcptr = (byte *)&bcaddr;
-		// Make the address a broadcast address
-		bcptr[3] = 255;
-
-		DEBUG(net, 6)("[NET][UDP] Broadcasting to %s", inet_ntoa(*(struct in_addr *)&bcaddr));
-
+	while (_broadcast_list[i] != 0) {
 		out_addr.sin_family = AF_INET;
 		out_addr.sin_port = htons(_network_server_port);
-		out_addr.sin_addr.s_addr = bcaddr;
+		out_addr.sin_addr.s_addr = _broadcast_list[i];
+
+		DEBUG(net, 6)("[NET][UDP] Broadcasting to %s", inet_ntoa(out_addr.sin_addr));
 
 		NetworkSendUDP_Packet(udp, p, &out_addr);
 
 		i++;
 	}
+
+	free(p);
 }
 
 
@@ -487,7 +538,7 @@ void NetworkUDPSearchGame(void)
 	_network_udp_broadcast = 300; // Stay searching for 300 ticks
 }
 
-NetworkGameList *NetworkUDPQueryServer(const byte* host, unsigned short port)
+NetworkGameList *NetworkUDPQueryServer(const char* host, unsigned short port)
 {
 	struct sockaddr_in out_addr;
 	Packet *p;
@@ -530,12 +581,12 @@ void NetworkUDPRemoveAdvertise(void)
 	Packet *p;
 
 	/* Check if we are advertising */
-	if (!_networking || !_network_server || !_network_udp_server || !_network_advertise)
+	if (!_networking || !_network_server || !_network_udp_server)
 		return;
 
 	/* check for socket */
 	if (_udp_master_socket == INVALID_SOCKET)
-		if (!NetworkUDPListen(&_udp_master_socket, 0, 0, false))
+		if (!NetworkUDPListen(&_udp_master_socket, _network_server_bind_ip, 0, false))
 			return;
 
 	DEBUG(net, 2)("[NET][UDP] Removing advertise..");
@@ -568,21 +619,26 @@ void NetworkUDPAdvertise(void)
 
 	/* check for socket */
 	if (_udp_master_socket == INVALID_SOCKET)
-		if (!NetworkUDPListen(&_udp_master_socket, 0, 0, false))
+		if (!NetworkUDPListen(&_udp_master_socket, _network_server_bind_ip, 0, false))
 			return;
 
-	/* Only send once in the 450 game-days (about 15 minutes) */
-	if (_network_advertise_retries == 0) {
-		if ( (_network_last_advertise_date + ADVERTISE_NORMAL_INTERVAL) > _date)
-			return;
+	if (_network_need_advertise) {
+		_network_need_advertise = false;
 		_network_advertise_retries = ADVERTISE_RETRY_TIMES;
+	} else {
+		/* Only send once every ADVERTISE_NORMAL_INTERVAL ticks */
+		if (_network_advertise_retries == 0) {
+			if ((_network_last_advertise_frame + ADVERTISE_NORMAL_INTERVAL) > _frame_counter)
+				return;
+			_network_advertise_retries = ADVERTISE_RETRY_TIMES;
+		}
+
+		if ((_network_last_advertise_frame + ADVERTISE_RETRY_INTERVAL) > _frame_counter)
+			return;
 	}
 
-	if ( (_network_last_advertise_date + ADVERTISE_RETRY_INTERVAL) > _date)
-		return;
-
 	_network_advertise_retries--;
-	_network_last_advertise_date = _date;
+	_network_last_advertise_frame = _frame_counter;
 
 	/* Find somewhere to send */
 	out_addr.sin_family = AF_INET;
