@@ -1,4 +1,4 @@
-/* $Id: openttd.c 3500 2006-01-31 22:16:15Z Darkvater $ */
+/* $Id: openttd.c 5488 2006-07-13 17:54:57Z truelight $ */
 
 #include "stdafx.h"
 #include "string.h"
@@ -141,7 +141,7 @@ static void showhelp(void)
 		"  -G seed             = Set random seed\n"
 		"  -n [ip#player:port] = Start networkgame\n"
 		"  -D                  = Start dedicated server\n"
-		#if !defined(__MORPHOS__) && !defined(__AMIGA__)
+		#if !defined(__MORPHOS__) && !defined(__AMIGA__) && !defined(WIN32)
 		"  -f                  = Fork into the background (dedicated only)\n"
 		#endif
 		"  -i                  = Force to use the DOS palette\n"
@@ -632,6 +632,7 @@ static void MakeNewEditorWorld(void)
 
 void StartupPlayers(void);
 void StartupDisasters(void);
+extern void StartupEconomy(void);
 
 /**
  * Start Scenario starts a new game based on a scenario.
@@ -669,6 +670,7 @@ static void StartScenario(void)
 	_opt.diff_level = _opt_newgame.diff_level;
 
 	// Inititalize data
+	StartupEconomy();
 	StartupPlayers();
 	StartupEngines();
 	StartupDisasters();
@@ -983,7 +985,8 @@ void GameLoop(void)
 
 #ifdef ENABLE_NETWORK
 	// Check for UDP stuff
-	NetworkUDPGameLoop();
+	if (_network_available)
+		NetworkUDPGameLoop();
 
 	if (_networking) {
 		// Multiplayer
