@@ -1,4 +1,4 @@
-/* $Id: vehicle_gui.c 3396 2006-01-12 15:52:18Z peter1138 $ */
+/* $Id: vehicle_gui.c 4669 2006-05-02 13:11:00Z Darkvater $ */
 
 #include "stdafx.h"
 #include "openttd.h"
@@ -431,7 +431,7 @@ static void train_engine_drawing_loop(int *x, int *y, int *pos, int *sel, Engine
 
 
 		colour = *sel == 0 ? 0xC : 0x10;
-		if (!(ENGINE_AVAILABLE && show_outdated && RailVehInfo(i)->power && e->railtype == railtype)) {
+		if (!(ENGINE_AVAILABLE && show_outdated && RailVehInfo(i)->power && IsCompatibleRail(e->railtype, railtype))) {
 			if (e->railtype != railtype || !(rvi->flags & RVI_WAGON) != is_engine ||
 				!HASBIT(e->player_avail, _local_player))
 				continue;
